@@ -1,7 +1,8 @@
 import warnings
 from collections import deque
-from typing import Dict, Iterable, Optional, Union
+from typing import Dict, Optional, Union
 
+import aesara.tensor as at
 from aesara import config
 from aesara.graph.basic import graph_inputs, io_toposort
 from aesara.graph.fg import FunctionGraph
@@ -17,17 +18,17 @@ from aeppl.opt import PreserveRVMappings, logprob_rewrites_db
 from aeppl.utils import rvs_to_value_vars
 
 
-def joint_logprob(
-    vars: Union[TensorVariable, Iterable[TensorVariable]],
+def factorized_joint_logprob(
     rv_values: Dict[TensorVariable, TensorVariable],
     warn_missing_rvs: bool = True,
     extra_rewrites: Optional[Union[GlobalOptimizer, LocalOptimizer]] = None,
     **kwargs,
-) -> TensorVariable:
-    r"""Create a graph representing the joint log-probability/measure of a graph.
+) -> Dict[TensorVariable, TensorVariable]:
+    r"""Create a map between variables and their log-probabilities such that the
+    sum is their joint log-probability.
 
-    The input `var` determines which graph is used and `rv_values` specifies
-    the resulting measure-space graph's input parameters.
+    The `rv_values` dictionary specifies a joint probability graph defined by
+    pairs of random variables and respective measure-space input parameters
 
     For example, consider the following
 
@@ -35,22 +36,22 @@ def joint_logprob(
 
         import aesara.tensor as at
 
-        Y_rv = at.random.normal(0, at.sqrt(sigma2_rv))
         sigma2_rv = at.random.invgamma(0.5, 0.5)
+        Y_rv = at.random.normal(0, at.sqrt(sigma2_rv))
 
     This graph for ``Y_rv`` is equivalent to the following hierarchical model:
 
     .. math::
 
-        Y \sim& \operatorname{N}(0, \sigma^2) \\
-        \sigma^2 \sim& \operatorname{InvGamma}(0.5, 0.5)
+        \sigma^2 \sim& \operatorname{InvGamma}(0.5, 0.5) \\
+        Y \sim& \operatorname{N}(0, \sigma^2)
 
-    If we create a value variable for ``Y_rv``, i.e. ``y = at.scalar("y")``,
-    the graph of ``joint_logprob(Y_rv, {Y_rv: y})`` is equivalent to the
-    conditional probability :math:`\log p(Y = y \mid \sigma^2)`.  If we specify
-    a value variable for ``sigma2_rv``, i.e. ``s = at.scalar("s2")``, then
-    ``joint_logprob(Y_rv, {Y_rv: y, sigma2_rv: s})`` yields the joint
-    log-probability
+    If we create a value variable for ``Y_rv``, i.e. ``y_vv = at.scalar("y")``,
+    the graph of ``factorized_joint_logprob({Y_rv: y_vv})`` is equivalent to the
+    conditional probability :math:`\log p(Y = y \mid \sigma^2)`, with a stochastic
+    ``sigma2_rv``. If we specify a value variable for ``sigma2_rv``, i.e.
+    ``s_vv = at.scalar("s2")``, then ``factorized_joint_logprob({Y_rv: y_vv, sigma2_rv: s_vv})``
+    yields the joint log-probability of the two variables.
 
     .. math::
 
@@ -60,12 +61,6 @@ def joint_logprob(
 
     Parameters
     ==========
-    vars
-        The graphs containing the stochastic/`RandomVariable` elements for
-        which we want to compute a joint log-probability.  This graph
-        effectively represents a statistical model.  When multiple elements are
-        given, their log-probabilities are summed, producing their joint
-        log-probability.
     rv_values
         A ``dict`` of variables that maps stochastic elements
         (e.g. `RandomVariable`\s) to symbolic `Variable`\s representing their
@@ -78,10 +73,12 @@ def joint_logprob(
         Extra rewrites to be applied (e.g. reparameterizations, transforms,
         etc.)
 
-    """
-    if isinstance(vars, TensorVariable):
-        vars = [vars]
+    Returns
+    =======
+    A ``dict`` that maps each value variable to the log-probability factor derived
+    from the respective `RandomVariable`.
 
+    """
     # Since we're going to clone the entire graph, we need to keep a map from
     # the old nodes to the new ones; otherwise, we won't be able to use
     # `rv_values`.
@@ -94,7 +91,7 @@ def joint_logprob(
     # to give good warnings when an unaccounted for `RandomVariable` is
     # encountered
     fgraph = FunctionGraph(
-        outputs=list(vars) + [k for k in rv_values.keys() if k not in vars],
+        outputs=list(rv_values.keys()),
         clone=True,
         memo=memo,
         copy_orphans=False,
@@ -126,7 +123,7 @@ def joint_logprob(
     # log-probability
     q = deque(fgraph.toposort())
 
-    logprob_var = None
+    logprob_vars = {}
 
     while q:
         node = q.popleft()
@@ -177,20 +174,55 @@ def joint_logprob(
                 if q_rv_var.name:
                     q_logprob_var.name = f"{q_rv_var.name}_logprob"
 
-                if logprob_var is None:
-                    logprob_var = q_logprob_var.sum()
-                else:
-                    logprob_var += q_logprob_var.sum()
+                # Recompute test values for the changes introduced by the
+                # replacements above.
+                if config.compute_test_value != "off":
+                    for node in io_toposort(
+                        graph_inputs((q_logprob_var,)),
+                        (q_logprob_var,),
+                    ):
+                        compute_test_value(node)
+
+                if q_rv_var in logprob_vars:
+                    raise ValueError(
+                        f"More than one logprob factor was assigned to the value var {q_rv_var}"
+                    )
+
+                logprob_vars[q_rv_var] = q_logprob_var
 
         else:
             raise NotImplementedError(
                 f"A measure/probability could not be derived for {node}"
             )
 
-    # Recompute test values for the changes introduced by the replacements
-    # above.
-    if config.compute_test_value != "off":
-        for node in io_toposort(graph_inputs((logprob_var,)), (logprob_var,)):
-            compute_test_value(node)
+    return logprob_vars
 
-    return logprob_var
+
+def joint_logprob(*args, sum: bool = True, **kwargs) -> Optional[TensorVariable]:
+    """Create a graph representing the joint log-probability/measure of a graph.
+
+    This function calls `factorized_joint_logprob` and returns the combined
+    log-probability factors as a single graph.
+
+    Parameters
+    ----------
+    sum: bool
+        If ``True`` each factor is collapsed to a scalar via ``sum`` before
+        being joined with the remaining factors. This may be necessary to
+        avoid incorrect broadcasting among independent factors.
+
+    """
+    logprob = factorized_joint_logprob(*args, **kwargs)
+    if not logprob:
+        return None
+    elif len(logprob) == 1:
+        logprob = tuple(logprob.values())[0]
+        if sum:
+            return at.sum(logprob)
+        else:
+            return logprob
+    else:
+        if sum:
+            return at.sum([at.sum(factor) for factor in logprob.values()])
+        else:
+            return at.add(*logprob.values())
